
public class TourCreator {

	public TourCreator(Minimizer minimizer){
		this.minimizer = minimizer;
	}
	private final Minimizer minimizer;
	
	
	void initialize(){
	    /* init arrays */
	    for (int i = 0; i < Tsp.MAX_NUM_TOURS; i++) {
	    	Tours[i] = new TourElement();
	    	PrioQ[i] = new PrioQElement();
	    }
	    
	    /* Initialize first tour */
	    Tours[0].prefix[0] = Tsp.StartNode;
	    Tours[0].conn = 1;
	    Tours[0].last = 0;
	    Tours[0].prefix_weight = 0;
	    calc_bound(0); /* Sets lower_bound */
	    
	    /* Initialize the priority queue structures */
	    PrioQ[1].index = 0;
	    PrioQ[1].priority = Tours[0].lower_bound;
	    PrioQLast = 1;
	    
	    /* Put all unused tours in the free tour stack */
	    for (int i = Tsp.MAX_NUM_TOURS - 1; i > 0; i--)
	    	TourStack[++TourStackTop] = i;
	}
	

	/*
	 *  Add up min edges connecting all unconnected vertixes (AHU p. 331-335)
	 *  At some point, we may want to use MST to get a lower bound. This
	 *  bound should be higher (and therefore more accurate) but will take
	 *  longer to compute. 
	 */
	/*static*/ int calc_bound(int curr_index) {
	int i, j, wt, wt1, wt2;
	TourElement curr = Tours[curr_index];
	
//	synchronized(TourLock) {
	    if (TspSolver.debug) 
		System.out.println("calc_bound");
	
	    /*
	     * wt1: the value of the edge with the lowest weight from the node
	     *	    we're testing to another unconnected node.
	     * wt2: the value of the edge with the second lowest weight
	     */
	
	    /* if we have one unconnected node */
	    if (curr.last == (Tsp.TspSize - 2)) {
		for (i = 0; i < Tsp.TspSize; i++) {
		    if ((curr.conn & (1<<i))==0) {
			/* we have found the one unconnected node */
			curr.prefix[Tsp.TspSize-1] = i;
			curr.prefix[Tsp.TspSize] = Tsp.StartNode;
		    
			/* add edges to and from the last node */
			curr.prefix_weight += weights[curr.prefix[Tsp.TspSize-2]][i] +
			    weights[i][curr.prefix[Tsp.StartNode]];
		    
			if (curr.prefix_weight < minimizer.getMinTourLen()) {
			    /* Store our new best path and its weight. */
			    minimizer.set_best(curr.prefix_weight, curr.prefix);
			}
		    
			/* De-allocate this tour so someone else can use it */
			curr.lower_bound = Tsp.BIGINT;
		    
			TourStack[++TourStackTop] = curr_index; /* Free tour. */
			return(Tsp.END_TOUR);
		    }
		}
	    }
	
	    curr.mst_weight = 0;
	
	    /*
	     * Add up the lowest weights for edges connected to vertices
	     * not yet used or at the ends of the current tour, and divide by two.
	     * This could be tweaked quite a bit.  For example:
	     *   (1) Check to make sure that using an edge would not make it
	     *       impossible for a vertex to have degree two.
	     *   (2) Check to make sure that the edge doesn't give some
	     *       vertex degree 3.
	     */
	
	    if (curr.last != Tsp.TspSize - 1) {
		for (i = 0; i < Tsp.TspSize; i++) {
		    if ((curr.conn & 1<<i)!=0) continue;
		    for (j = 0, wt1 = wt2 = Tsp.BIGINT; j < Tsp.TspSize; j++) {
			/* Ignore j's that are not connected to i (global->weights[i][j]==0), */
			/* or that are already in the tour and aren't either the      */
			/* first or last node in the current tour.		      */
			wt = weights[i][j];
			if ((wt==0) || (((curr.conn&(1<<i))!=0)&&(j != curr.prefix[0])&&
					(j != curr.prefix[curr.last])))
			    continue;
		    
			/* Might want to check that edges go to unused vertices */
			if (wt < wt1) {
			    wt2 = wt1;
			    wt1 = wt;
			} else if (wt < wt2) wt2 = wt;
		    }
	
		    /* At least three unconnected nodes? */
		    if (wt2 != Tsp.BIGINT) curr.mst_weight += (wt1 + wt2) >> 1;
		    /* Exactly two unconnected nodes? */
		    else if (wt1 != Tsp.BIGINT) curr.mst_weight += wt1;
		}
		curr.mst_weight += 1;
	    }
	    curr.lower_bound = curr.mst_weight + curr.prefix_weight;
	    return(curr_index);
//	}
	}
	/*static*/ int get_tour(int curr) {
	
	if (TspSolver.debug) 
	    System.out.println("get_tour");
	
//	synchronized(TourLock) {
	    if (curr != -1) TourStack[++TourStackTop] = curr;
	    curr = find_solvable_tour();
//	}
	return(curr);
	}
	/*
	 * find_solvable_tour():
	 *
	 * Used by both the normal TSP program (called by get_tour()) and
	 * the RPC server (called by RPCserver()) to return the next solvable
	 * (sufficiently short) tour.
	 *
	 */
	/*static*/ int find_solvable_tour() {
	int curr, i, left, right, child, index;
	int priority, last;
	PrioQElement pptr, cptr, lptr, rptr;
	
//	synchronized(TourLock) {
	    if (TspSolver.debug) 
		System.out.println("find_solvable_tour");
	
	    if (Done != 0) { 
		if (TspSolver.debug) 
		    System.out.println("(" + Thread.currentThread().getName() + ") - done"); 
		return -1; 
	    }
	
	    for ( ; PrioQLast != 0; ) {
		pptr = PrioQ[1];
		curr = pptr.index;
		if (pptr.priority >= minimizer.getMinTourLen()) {
		    /* We're done -- there's no way a better tour could be found. */
		    if (TspSolver.debug) {
			System.out.print("\t" + Thread.currentThread().getName() + ": ");		
			TspSolver.MakeTourString(Tours[curr].last, Tours[curr].prefix);
		    }
		    Done = 1;
		    return -1;
		}
	
		/* Bubble everything maintain the priority queue's heap invariant. */
		/* Move last element to root position. */
		cptr = PrioQ[PrioQLast];
		pptr.index  = cptr.index;
		pptr.priority = cptr.priority;
		PrioQLast--;
	
		/* Push previous last element down tree to restore heap structure. */
		for (i = 1; i <= (PrioQLast >> 1); ) {
		
		    /* Find child with lowest priority. */
		    left  = LEFT_CHILD(i);
		    right = RIGHT_CHILD(i);
	    
		    lptr = PrioQ[left];
		    rptr = PrioQ[right];
		
		    if (left == PrioQLast || less_than(lptr,rptr)) {
			child = left;
			cptr = lptr;
		    } else {
			child = right;
			cptr = rptr;
		    }
	
	
		    /* Exchange parent and child, if necessary. */
		    if (less_than(cptr,pptr)) {
			/* PrioQ[child] has lower prio than its parent - switch 'em. */
			index = pptr.index;
			priority = pptr.priority;
			pptr.index = cptr.index;
			pptr.priority = cptr.priority;
			cptr.index = index;
			cptr.priority = priority;
			i = child;
			pptr = cptr;
		    } else break;
		}
	    
		last = Tours[curr].last;
	    
		/* If we're within `NodesFromEnd' nodes of a complete tour, find
		   minimum solutions recursively.  Else, split the tour. */
		if (last < Tsp.TspSize || last < 1) {
		    if (last >= (Tsp.TspSize - Tsp.NodesFromEnd - 1)) return(curr);
		    else split_tour(curr);	/* The current tour is too long, */
		}				/* to solve now, break it up.	 */
		else {
		    /* Bogus tour index. */
		    if (TspSolver.debug) {
			System.out.print("\t" + Thread.currentThread().getName() + ": ");
			TspSolver.MakeTourString(Tsp.TspSize, Tours[curr].prefix);
		    }
		}
		TourStack[++TourStackTop] = curr; /* Free tour. */
	
	    }
	    /* Ran out of candidates - DONE! */
	    Done = 1;
	    return(-1);
//	}
	}
	/*
	 * split_tour():
	 *
	 *  Break current tour into subproblems, and stick them all in the priority
	 *  queue for later evaluation.
	 *
	 */
	/*static*/ void split_tour(int curr_ind) {
	int n_ind, last_node, wt;
	int i, pq, parent, index, priority;
	TourElement curr;
	PrioQElement cptr, pptr;
	
//	synchronized(TourLock) {
	    if (TspSolver.debug) 
		System.out.println("split_tour");
	
	    curr = Tours[curr_ind];
	
	    if (TspSolver.debug)
		TspSolver.MakeTourString(curr.last, curr.prefix);
	
	    /* Create a tour and add it to the priority Q for each possible
	       path that can be derived from the current path by adding a single
	       node while staying under the current minimum tour length. */
	
	    if (curr.last != (Tsp.TspSize - 1)) {
		boolean t1, t2, t3;
		TourElement new_;
	    
		last_node = curr.prefix[curr.last];
		for (i = 0; i < Tsp.TspSize; i++) {
		    /*
		     * Check: 1. Not already in tour
		     *	      2. Edge from last entry to node in graph
		     *	and   3. Weight of new partial tour is less than cur min.
		     */
		    wt = weights[last_node][i];
		    t1 = ((curr.conn & (1<<i)) == 0);
		    t2 = (wt != 0);
		    t3 = (curr.lower_bound + wt) <= minimizer.getMinTourLen();
		    if (t1 && t2 && t3) {
			if ((n_ind = new_tour(curr_ind, i)) == Tsp.END_TOUR) {
			    continue;
			}
			/*
			 * If it's shorter than the current best tour, or equal
			 * to the current best tour and we're reporting all min
			 * length tours, put it on the priority q.
			 */
			new_ = Tours[n_ind];
		    
			if (PrioQLast >= Tsp.MAX_NUM_TOURS-1) {
			    System.out.println("pqLast "+PrioQLast);
			    System.exit(-1);
			}
	
			if (TspSolver.debug)
			    TspSolver.MakeTourString(new_.last, new_.prefix);
		    
			pq = ++PrioQLast;
			cptr = PrioQ[pq];
			cptr.index = n_ind;
			cptr.priority = new_.lower_bound;
		    
			/* Bubble the entry up to the appropriate level to maintain
			   the invariant that a parent is less than either of it's
			   children. */
			for (parent = pq >> 1, pptr = PrioQ[parent];
			     (pq > 1) && less_than(cptr, pptr);
			     pq = parent,  cptr = pptr, parent = pq >> 1, pptr = PrioQ[parent]) {
			    /* PrioQ[pq] lower priority than parent -> SWITCH THEM. */
			    index = cptr.index;
			    priority = cptr.priority;
			    cptr.index = pptr.index;
			    cptr.priority = pptr.priority;
			    pptr.index = index;
			    pptr.priority = priority;
			}
		    } else if (TspSolver.debug) 
			System.out.println(" [" + curr.lower_bound + " + " + wt + " > " + minimizer.getMinTourLen());
		}
	    }
//	}
	}
	/*
	 * new_tour():
	 *
	 *    Create a new tour structure given an existing tour structure and
	 *  the next edge to add to the tour.  Returns the index of the new structure.
	 *
	 */
	/*static*/ int new_tour(int prev_index, int move) {
	    int index = 0;
	    int i;
	    TourElement curr, prev;
	    
//	    synchronized(TourLock) {
		if (TspSolver.debug) 
		    System.out.println("new_tour");
	
		if (TourStackTop >= 0) 
		    index = TourStack[TourStackTop--];
		else {
		    System.out.println("TourStackTop: " + TourStackTop);
		    System.exit(-1);
		}
	
		curr = Tours[index];
		prev = Tours[prev_index];
	
		for (i = 0; i < Tsp.TspSize; i++) {
		    curr.prefix[i] = prev.prefix[i];
		    curr.conn = prev.conn;
		}
		curr.last = prev.last;
		curr.prefix_weight = prev.prefix_weight + 
		    weights[curr.prefix[curr.last]][move];
		curr.prefix[++curr.last] = move;
	
		if (TspSolver.debug) 
		    TspSolver.MakeTourString(curr.last, curr.prefix);
	
		curr.conn |= 1 << move;
	
		return calc_bound(index);
//	    }
	}

	/*static*/ boolean less_than(PrioQElement x, PrioQElement y) {
	return ((x.priority  < y.priority) || 
		(x.priority == y.priority) && 
		(Tours[x.index].last > Tours[y.index].last));
	}
	/*
	 * DumpPrioQ():
	 *
	 * Dump the contents of PrioQ in some user-readable format (for debugging).
	 *
	 */
	/*static*/ void DumpPrioQ() {
	int pq, ind;
	
	System.out.println("DumpPrioQ");
	for (pq = 1; pq <= PrioQLast; pq++) {
	    ind = PrioQ[pq].index;
	    System.out.print(ind+"("+PrioQ[pq].priority+"):\t");
	    if ((LEFT_CHILD(pq)<PrioQLast) && (PrioQ[pq].priority>PrioQ[LEFT_CHILD(pq)].priority))
		System.out.print(" left child wrong! ");
	    if ((LEFT_CHILD(pq)<PrioQLast) && (PrioQ[pq].priority>PrioQ[RIGHT_CHILD(pq)].priority)) 
		System.out.print(" right child wrong! ");
	    TspSolver.MakeTourString(Tours[ind].last, Tours[ind].prefix);
	}
	}
	/*static*/ int LEFT_CHILD(int x) {
	return (x<<1);
	}
	/*static*/ int RIGHT_CHILD(int x) {
	return (x<<1)+1;
	}
	
	public TourElement[] getTours(){
		return Tours;
	}
	
	public int[][] getWeights(){
		return weights;
	}
	
	public void setTourStackTop(int val){
		TourStackTop = val;
	}
	
	/*atomicset(TC)*/
	/*atomic(TC)*/ private int[] TourStack = new int[Tsp.MAX_NUM_TOURS];
	/*atomic(TC)*/ private int TourStackTop;
	/*atomic(TC)*/ private TourElement[] Tours = new TourElement[Tsp.MAX_NUM_TOURS];
	/*atomic(TC)*/ private int[][] weights = new int[Tsp.MAX_TOUR_SIZE][Tsp.MAX_TOUR_SIZE];
//	/*atomic(TC)*/ Integer TourLock = new Integer(0);
	/*atomic(TC)*/ private int Done;
	/*atomic(TC)*/ private int PrioQLast;
	/*atomic(TC)*/ private PrioQElement[] PrioQ = new PrioQElement[Tsp.MAX_NUM_TOURS];

}
