/*
 * Copyright (C) 2000 by ETHZ/INF/CS
 * All rights reserved
 * 
 * @version $Id: TspSolver.java,v 1.1 2010/05/27 18:59:40 cpa Exp $
 * @author Florian Schneider
 */

public class TspSolver extends Thread {

	
	public TspSolver(TourCreator tourCreator, Minimizer minimizer){
		this.tourCreator = tourCreator;
		this.minimizer = minimizer;
	}

    private final TourCreator tourCreator;
    private final Minimizer minimizer;
    
    static final boolean debug = Tsp.debug;
    
    static Integer barrier = new Integer(0);
    /* non-static member variables, used by recusive_solve and visit_nodes */
    int CurDist, PathLen;
    int[] Visit = new int[Tsp.MAX_TOUR_SIZE];
    int[] Path = new int[Tsp.MAX_TOUR_SIZE];
    int visitNodes;

    public void run() {
	Worker();
    }

    void Worker() {
	int curr = -1;
	for ( ; ; ) {
	    /* Continuously get a tour and split. */
	    curr = tourCreator.get_tour(curr);
	    if (curr < 0) {
		if (debug) 
		    System.out.println("Worker: curr = " + curr); 
		break;
	    }
	    recursive_solve(curr);
	    if (debug) 
		tourCreator.DumpPrioQ();
	}
    }

    static void MakeTourString(int len, int[] path) {
	int i;
	String tour_str="";

	for (i=0; i < len; i++) {
	    System.out.print(path[i]+" - ");
	    if (path[i] >= 10) System.out.print("   ");
	    else System.out.print("  ");
	}
	System.out.println(path[i]);
    }


    /*
     *   recursive_solve(curr_ind)
     *
     *	We're now supposed to recursively try all possible solutions
     *	starting with the current tour.  We do this by copying the
     *	state to local variables (to avoid unneeded conflicts) and
     *	calling visit_nodes to do the actual recursive solution.
     *
     */
    void recursive_solve(int index) {
	int i, j;
	TourElement curr = tourCreator.Tours[index];

	if (debug) 
	    System.out.println("recursive_solve");

	CurDist = curr.prefix_weight;
	PathLen = curr.last + 1;
	
	for (i = 0; i < Tsp.TspSize; i++) Visit[i] = 0;
	
	for (i = 0; i < PathLen; i++) {
	    Path[i] = curr.prefix[i];
	    Visit[Path[i]] = 1;
	}

	if (PathLen > Tsp.TspSize) {
	    System.out.println("Pathlen: "+ PathLen);
	    System.exit(0);
	}
	
	if (CurDist == 0 || debug) {
	    if (debug) 
		System.out.print("\t" + Thread.currentThread().getName() + ": Tour " + index + " is ");
	    for (i = 0; i < PathLen-1; i++) {
		System.out.print(Path[i]+" - ");
	    }
	    if (debug) { 
		System.out.println(Path[i]);
		System.out.println("\t" + Thread.currentThread().getName() + ": Cur: " + CurDist + ", Min " + minimizer.MinTourLen + 
				   ", Len:" + (PathLen - 1) + ", Sz: " + Tsp.TspSize + "\n");
	    }
	}
	
	visit_nodes(Path[PathLen-1]);
    }

    /*
     *   visit_nodes()
     *
     *       Exhaustively visits each node to find Hamilton cycle.
     *       Assumes that search started at node from.
     *
     */
    void visit_nodes(int from) {
	int i;
	int dist, last;

	visitNodes++;	
	
	for (i = 1; i < Tsp.TspSize; i++) {
	    if (Visit[i]!=0) continue;	/* Already visited. */
	    if ((dist = tourCreator.weights[from][i]) == 0) 
		continue; /* Not connected. */
	    if (CurDist + dist > minimizer.MinTourLen) 
		continue; /* Path too long. */
	    
	    /* Try next node. */
	    Visit[i] = 1;
	    Path[PathLen++] = i;
	    CurDist += dist;
	    
	    if (PathLen == Tsp.TspSize) {
		/* Visiting last node - determine if path is min length. */
		if ((last = tourCreator.weights[i][Tsp.StartNode]) != 0 &&
		    (CurDist += last) < minimizer.MinTourLen) {
		    minimizer.set_best(CurDist, Path);
		}
		CurDist -= last;
	    } /* if visiting last node */
	    else if (CurDist < minimizer.MinTourLen) 
		visit_nodes(i);	/* Visit on. */
	    
	    /* Remove current try and loop again at this level. */
	    CurDist -= dist;
	    PathLen--;
	    Visit[i] = 0;
	}
    }
}

		     




